﻿/*
Copyright (c) 2003-2013, CKSource - Frederico Knabben. All rights reserved.
For licensing, see LICENSE.html or http://ckeditor.com/license
*/
define("ckeditor/plugins/eptable/dialogs/table", [
	"jquery",
	"ckeditor/ckeditor",
	"$dict!ep/dict",

	"ep/ui/colortool",
	"ep/ui/colorpicker"
], function ($, CKEDITOR, epDict) {

	(function()
	{
		var defaultToPixel = CKEDITOR.tools.cssLength,
			cssTableWidthBorder = "editor_table_with_border",
			cssTdWidthBorder = "editor_td_with_border";

		var commitValue = function( data )
		{
			var id = this.id;
			if ( !data.info )
				data.info = {};
			data.info[id] = this.getValue();
		};

		function tableColumns( table )
		{
			var cols = 0, maxCols = 0;
			for ( var i = 0, row, rows = table.$.rows.length; i < rows; i++ )
			{
				row = table.$.rows[ i ], cols = 0;
				for ( var j = 0, cell, cells = row.cells.length; j < cells; j++ )
				{
					cell = row.cells[ j ];
					cols += cell.colSpan;
				}

				cols > maxCols && ( maxCols = cols );
			}

			return maxCols;
		}


		// Whole-positive-integer validator.
		function validatorNum( msg )
		{
			return function()
			{
				var value = this.getValue(),
					pass = !!( CKEDITOR.dialog.validate.integer()( value ) && value > 0 );

				if ( !pass )
				{
					alert( msg );
					this.select();
				}

				return pass;
			};
		}

		// Update Style properties for border and border-color
		function updateBorder(item, info, className){
			if(info.txtBorder){
				item.setStyle('border-width', (isNaN(parseInt(info.txtBorder))? "0px" : info.txtBorder+"px"));
				item.setStyle('border-style', "solid");
				item.setStyle('border-color', (info.txtBorderColor? info.txtBorderColor : "transparent"));

				if(!item.hasClass(className)){
					item.addClass(className);
				}
			}else{
				item.removeStyle('border-width');
				item.removeStyle('border-style');
				item.removeStyle('border-color');

				item.removeClass(className);
			}
		}

		function tableDialog( editor, command )
		{
			var makeElement = function( name )
				{
					return new CKEDITOR.dom.element( name, editor.document );
				};

			var dialogadvtab = editor.plugins.dialogadvtab;

			return {
				title : editor.lang.table.title,
				minWidth : 310,
				minHeight : CKEDITOR.env.ie ? 310 : 280,

				onLoad : function()
				{
					var dialog = this,
						styles = dialog.getContentElement( 'advanced', 'advStyles' ),
						txtBackgroundColor = dialog.getContentElement( 'info', 'txtBackgroundColor' ),
						$backgroundColorInput = $("#"+txtBackgroundColor.domId).find('input'),
						$backgroundColorLabel = $("#"+txtBackgroundColor.domId).find('label'),
						txtBorderColor = dialog.getContentElement( 'info', 'txtBorderColor' ),
						$borderColorInput = $("#"+txtBorderColor.domId).find('input'),
						$borderColorLabel = $("#"+txtBorderColor.domId).find('label');

					if ( styles )
					{
						styles.on( 'change', function( evt )
							{
								// Synchronize width value.
								var width = this.getStyle( 'width', '' ),
									txtWidth = dialog.getContentElement( 'info', 'txtWidth' );

								txtWidth && txtWidth.setValue( width, true );

								// Synchronize height value.
								var height = this.getStyle( 'height', '' ),
									txtHeight = dialog.getContentElement( 'info', 'txtHeight' );

								txtHeight && txtHeight.setValue( height, true );

								// Synchronize border-width value.
								var borderWidth = this.getStyle( 'border-width', '' ),
									txtBorder = dialog.getContentElement( 'info', 'txtBorder' );

								txtBorder && txtBorder.setValue( "2"); //borderWidth, true );

								// Synchronize border-color value.
								var borderColor = this.getStyle( 'border-color', '' );

								txtBorderColor && txtBorderColor.setValue( borderColor, true );

								// Synchronize border-color value.
								var backgroundColor = this.getStyle( 'background-color', '' );

								txtBackgroundColor && txtBackgroundColor.setValue( backgroundColor, true );
							});
					}

					$("#"+txtBackgroundColor.domId).css({
						'position': 'relative'
					});
					$backgroundColorLabel.css({
						'margin-left': '40px'
					});
					$backgroundColorInput
						.css({
							opacity: 0
						})
						.uiColortool({
							orientation: "top right",
							//autoOpen: true,
							transparent: true
						});

					$("#"+txtBorderColor.domId).css({
						'position': 'relative'
					});
					$borderColorLabel.css({
						'margin-left': '40px'
					});
					$borderColorInput
						.css({
							opacity: 0
						})
						//.addClass('Opacity0')
						.uiColortool({
							orientation: "top right",
							transparent: true
						});
				},

				onShow : function()
				{
					// Detect if there's a selected table.
					var selection = editor.getSelection(),
						ranges = selection.getRanges(),
						selectedTable = null;

					var rowsInput = this.getContentElement( 'info', 'txtRows' ),
						colsInput = this.getContentElement( 'info', 'txtCols' ),
						widthInput = this.getContentElement( 'info', 'txtWidth' ),
						heightInput = this.getContentElement( 'info', 'txtHeight' ),
						borderInput = this.getContentElement( 'info', 'txtBorder' ),
						borderColorInput = this.getContentElement( 'info', 'txtBorderColor' ),
						backgroundColorInput = this.getContentElement( 'info', 'txtBackgroundColor' ),
						backgroundColorPicker = $("#"+backgroundColorInput.domId).find('input').data('uiUiColortool'),
						borderColorPicker = $("#"+borderColorInput.domId).find('input').data('uiUiColortool'),
						currentStyle;

					if ( command == 'eptableProperties' )
					{
						if ( ( selectedTable = selection.getSelectedElement() ) )
							selectedTable = selectedTable.getAscendant( 'table', true );
						else if ( ranges.length > 0 )
						{
							// Webkit could report the following range on cell selection (#4948):
							// <table><tr><td>[&nbsp;</td></tr></table>]
							if ( CKEDITOR.env.webkit )
								ranges[ 0 ].shrink( CKEDITOR.NODE_ELEMENT );

							var rangeRoot = ranges[0].getCommonAncestor( true );
							selectedTable = rangeRoot.getAscendant( 'table', true );
						}

						// Save a reference to the selected table, and push a new set of default values.
						this._.selectedElement = selectedTable;
					}

					// Enable or disable the row, cols, width fields.
					if ( selectedTable )
					{
						this.setupContent( selectedTable );
						rowsInput && rowsInput.disable();
						colsInput && colsInput.disable();
					}
					else
					{
						rowsInput && rowsInput.enable();
						colsInput && colsInput.enable();
					}

					// set already existing border and background values
					if ( selectedTable ){
						borderInput.setValue(parseInt(selectedTable.getStyle('border-width')));

						currentStyle = selectedTable.getStyle('border-color');
						borderColorInput.setValue(currentStyle);
						borderColorPicker.colorSelect(currentStyle);
						borderColorPicker._setColorTool(currentStyle);

						currentStyle = selectedTable.getStyle('background-color');
						backgroundColorInput.setValue(currentStyle);
						backgroundColorPicker.colorSelect(currentStyle);
						backgroundColorPicker._setColorTool(currentStyle);
					}

					// Call the onChange method for the widht and height fields so
					// they get reflected into the Advanced tab.
					widthInput && widthInput.onChange();
					heightInput && heightInput.onChange();
					borderInput && borderInput.onChange();
					borderColorInput && borderColorInput.onChange();
					backgroundColorInput && backgroundColorInput.onChange();
				},
				onOk : function()
				{
					var selection = editor.getSelection(),
						bms = this._.selectedElement && selection.createBookmarks();

					var table = this._.selectedElement || makeElement( 'table' ),
						me = this,
						data = {};

					this.commitContent( data, table );

					if ( data.info )
					{
						var info = data.info;

						// Generate the rows and cols.
						if ( !this._.selectedElement )
						{
							var tbody = table.append( makeElement( 'tbody' ) ),
								rows = parseInt( info.txtRows, 10 ) || 0,
								cols = parseInt( info.txtCols, 10 ) || 0;

							for ( var i = 0 ; i < rows ; i++ )
							{
								var row = tbody.append( makeElement( 'tr' ) );
								for ( var j = 0 ; j < cols ; j++ )
								{
									var cell = row.append( makeElement( 'td' ) );

									updateBorder(cell, info, cssTdWidthBorder);

									if ( !CKEDITOR.env.ie ){
										// add special handling if only one cell and one row has to be inserted
										// (otherwise the table disappears when inserting a gadget or an image)
										if(rows===1 && cols===1){
											cell.$.innerHTML = "&nbsp;";
										}
										cell.append( makeElement( 'br' ) );
									}
								}
							}
						}
						// Update TDs
						else{
							var tdList = this._.selectedElement.getElementsByTag('td'),
								tdLength = tdList.$.length,
								cIndex,
								tdItem;

							for(cIndex=0; cIndex<tdLength; cIndex++){
								tdItem = tdList.getItem(cIndex);
								updateBorder(tdItem, info, cssTdWidthBorder);
							}
						}

						// Modify the table headers. Depends on having rows and cols generated
						// correctly so it can't be done in commit functions.

						// Should we make a <thead>?
						var headers = info.selHeaders;
						if ( !table.$.tHead && ( headers == 'row' || headers == 'both' ) )
						{
							var thead = new CKEDITOR.dom.element( table.$.createTHead() );
							tbody = table.getElementsByTag( 'tbody' ).getItem( 0 );
							var theRow = tbody.getElementsByTag( 'tr' ).getItem( 0 );

							// Change TD to TH:
							for ( i = 0 ; i < theRow.getChildCount() ; i++ )
							{
								var th = theRow.getChild( i );
								// Skip bookmark nodes. (#6155)
								if ( th.type == CKEDITOR.NODE_ELEMENT && !th.data( 'cke-bookmark' ) )
								{
									th.renameNode( 'th' );
									th.setAttribute( 'scope', 'col' );
								}
							}
							thead.append( theRow.remove() );
						}

						if ( table.$.tHead !== null && !( headers == 'row' || headers == 'both' ) )
						{
							// Move the row out of the THead and put it in the TBody:
							thead = new CKEDITOR.dom.element( table.$.tHead );
							tbody = table.getElementsByTag( 'tbody' ).getItem( 0 );

							var previousFirstRow = tbody.getFirst();
							while ( thead.getChildCount() > 0 )
							{
								theRow = thead.getFirst();
								for ( i = 0; i < theRow.getChildCount() ; i++ )
								{
									var newCell = theRow.getChild( i );
									if ( newCell.type == CKEDITOR.NODE_ELEMENT )
									{
										newCell.renameNode( 'td' );
										newCell.removeAttribute( 'scope' );
									}
								}
								theRow.insertBefore( previousFirstRow );
							}
							thead.remove();
						}

						// Should we make all first cells in a row TH?
						if ( !this.hasColumnHeaders && ( headers == 'col' || headers == 'both' ) )
						{
							for ( row = 0 ; row < table.$.rows.length ; row++ )
							{
								newCell = new CKEDITOR.dom.element( table.$.rows[ row ].cells[ 0 ] );
								newCell.renameNode( 'th' );
								newCell.setAttribute( 'scope', 'row' );
							}
						}

						// Should we make all first TH-cells in a row make TD? If 'yes' we do it the other way round :-)
						if ( ( this.hasColumnHeaders ) && !( headers == 'col' || headers == 'both' ) )
						{
							for ( i = 0 ; i < table.$.rows.length ; i++ )
							{
								row = new CKEDITOR.dom.element( table.$.rows[i] );
								if ( row.getParent().getName() == 'tbody' )
								{
									newCell = new CKEDITOR.dom.element( row.$.cells[0] );
									newCell.renameNode( 'td' );
									newCell.removeAttribute( 'scope' );
								}
							}
						}

						// Set the width and height.
						info.txtHeight ? table.setStyle( 'height', info.txtHeight ) : table.removeStyle( 'height' );
						info.txtWidth ? table.setStyle( 'width', info.txtWidth ) : table.removeStyle( 'width' );

						updateBorder(table, info, cssTableWidthBorder);

						if(info.txtBackgroundColor){
							table.setStyle('background-color', info.txtBackgroundColor);
						}else{
							table.removeStyle('background-color');
						}

						if ( !table.getAttribute( 'style' ) )
							table.removeAttribute( 'style' );
					}

					// Insert the table element if we're creating one.
					if ( !this._.selectedElement )
					{
						editor.insertElement( table );
						// Override the default cursor position after insertElement to place
						// cursor inside the first cell (#7959), IE needs a while.
						setTimeout( function()
							{
								var firstCell = new CKEDITOR.dom.element( table.$.rows[ 0 ].cells[ 0 ] );
								var range = new CKEDITOR.dom.range( editor.document );
								range.moveToPosition( firstCell, CKEDITOR.POSITION_AFTER_START );
								range.select( 1 );
							}, 0 );
					}
					// Properly restore the selection, (#4822) but don't break
					// because of this, e.g. updated table caption.
					else
						try { selection.selectBookmarks( bms ); } catch( er ){}
				},
				contents : [
					{
						id : 'info',
						label : editor.lang.table.title,
						elements :
						[
							{
								type : 'hbox',
								widths : [ null, null ],
								styles : [ 'vertical-align:top' ],
								children :
								[
									{
										type : 'vbox',
										padding : 0,
										children :
										[
											{
												type : 'text',
												id : 'txtRows',
												'default' : 3,
												label : editor.lang.table.rows,
												required : true,
												controlStyle : 'width:5em',
												validate : validatorNum( editor.lang.table.invalidRows ),
												setup : function( selectedElement )
												{
													this.setValue( selectedElement.$.rows.length );
												},
												commit : commitValue
											},
											{
												type : 'text',
												id : 'txtCols',
												'default' : 2,
												label : editor.lang.table.columns,
												required : true,
												controlStyle : 'width:5em',
												validate : validatorNum( editor.lang.table.invalidCols ),
												setup : function( selectedTable )
												{
													this.setValue( tableColumns( selectedTable ) );
												},
												commit : commitValue
											},
											{
												type : 'html',
												html : '&nbsp;'
											},
											{
												type : 'text',
												id : 'txtBorder',
												'default' : 1,
												label : editor.lang.table.border,
												controlStyle : 'width:3em',
												//getValue : defaultToPixel,
												//validate : CKEDITOR.dialog.validate.cssLength( editor.lang.common.invalidCssLength.replace( '%1', editor.lang.common.height ) ),
												validate : CKEDITOR.dialog.validate['number']( editor.lang.table.invalidBorder ),
												onChange : function()
												{
													var styles = this.getDialog().getContentElement( 'advanced', 'advStyles' );
													styles && styles.updateStyle( 'border-width', this.getValue() );
												},
												setup : function( selectedTable )
												{
													this.setValue( selectedTable.getAttribute( 'border' ) || '' );
												},
												commit : commitValue
											},
											{
												type : 'select',
												id : 'selHeaders',
												'default' : '',
												label : editor.lang.table.headers,
												items :
												[
													[ editor.lang.table.headersNone, '' ],
													[ editor.lang.table.headersRow, 'row' ],
													[ editor.lang.table.headersColumn, 'col' ],
													[ editor.lang.table.headersBoth, 'both' ]
												],
												setup : function( selectedTable )
												{
													// Fill in the headers field.
													var dialog = this.getDialog();
													dialog.hasColumnHeaders = true;

													// Check if all the first cells in every row are TH
													for ( var row = 0 ; row < selectedTable.$.rows.length ; row++ )
													{
														// If just one cell isn't a TH then it isn't a header column
														var headCell = selectedTable.$.rows[row].cells[0];
														if ( headCell && headCell.nodeName.toLowerCase() != 'th' )
														{
															dialog.hasColumnHeaders = false;
															break;
														}
													}

													// Check if the table contains <thead>.
													if ( ( selectedTable.$.tHead !== null) )
														this.setValue( dialog.hasColumnHeaders ? 'both' : 'row' );
													else
														this.setValue( dialog.hasColumnHeaders ? 'col' : '' );
												},
												commit : commitValue
											},
											{
												id : 'cmbAlign',
												type : 'select',
												'default' : '',
												label : editor.lang.common.align,
												items :
												[
													[ editor.lang.common.notSet , ''],
													[ editor.lang.common.alignLeft , 'left'],
													[ editor.lang.common.alignCenter , 'center'],
													[ editor.lang.common.alignRight , 'right']
												],
												setup : function( selectedTable )
												{
													this.setValue( selectedTable.getAttribute( 'align' ) || '' );
												},
												commit : function( data, selectedTable )
												{
													if ( this.getValue() )
														selectedTable.setAttribute( 'align', this.getValue() );
													else
														selectedTable.removeAttribute( 'align' );
												}
											}
										]
									},
									{
										type : 'vbox',
										padding : 0,
										children :
										[
											{
												type : 'hbox',
												widths : [ '5em' ],
												children :
												[
													{
														type : 'text',
														id : 'txtWidth',
														controlStyle : 'width:5em',
														label : editor.lang.common.width,
														title : editor.lang.common.cssLengthTooltip,
														'default' : 500,
														getValue : defaultToPixel,
														validate : CKEDITOR.dialog.validate.cssLength( editor.lang.common.invalidCssLength.replace( '%1', editor.lang.common.width ) ),
														onChange : function()
														{
															var styles = this.getDialog().getContentElement( 'advanced', 'advStyles' );
															styles && styles.updateStyle( 'width', this.getValue() );
														},
														setup : function( selectedTable )
														{
															var val = selectedTable.getStyle( 'width' );
															val && this.setValue( val );
														},
														commit : commitValue
													}
												]
											},
											{
												type : 'hbox',
												widths : [ '5em' ],
												children :
												[
													{
														type : 'text',
														id : 'txtHeight',
														controlStyle : 'width:5em',
														label : editor.lang.common.height,
														title : editor.lang.common.cssLengthTooltip,
														'default' : '',
														getValue : defaultToPixel,
														validate : CKEDITOR.dialog.validate.cssLength( editor.lang.common.invalidCssLength.replace( '%1', editor.lang.common.height ) ),
														onChange : function()
														{
															var styles = this.getDialog().getContentElement( 'advanced', 'advStyles' );
															styles && styles.updateStyle( 'height', this.getValue() );
														},

														setup : function( selectedTable )
														{
															var val = selectedTable.getStyle( 'height' );
															val && this.setValue( val );
														},
														commit : commitValue
													}
												]
											},
											{
												type : 'html',
												html : '&nbsp;'
											},
											{
												type : 'text',
												id : 'txtCellPad',
												controlStyle : 'width:3em',
												label : editor.lang.table.cellPad,
												'default' : 1,
												validate : CKEDITOR.dialog.validate.number( editor.lang.table.invalidCellPadding ),
												setup : function( selectedTable )
												{
													this.setValue( selectedTable.getAttribute( 'cellPadding' ) || '' );
												},
												commit : function( data, selectedTable )
												{
													if ( this.getValue() )
														selectedTable.setAttribute( 'cellPadding', this.getValue() );
													else
														selectedTable.removeAttribute( 'cellPadding' );
												}
											},
											{
												type : 'html',
												html : '&nbsp;<br />'
											},
											{
												type : 'text',
												id : 'txtBorderColor',
												'default' : '#000000',
												label : epDict.translate("BorderColor"),
												controlStyle : 'width:5em',
												className: 'epBoderColorCKE',
												onChange : function()
												{
													var styles = this.getDialog().getContentElement( 'advanced', 'advStyles' );
													styles && styles.updateStyle( 'border-color', this.getValue() );
												},
												setup : function( selectedTable )
												{
													this.setValue( selectedTable.getStyle( 'border-color' ) || '' );
												},
												commit : commitValue
											},
											{
												type : 'text',
												id : 'txtBackgroundColor',
												'default' : '#efefef',
												label : epDict.translate("BackgroundColor"),
												controlStyle : 'width:5em',
												className: 'epBackgroundColorCKE',
												onChange : function()
												{
													var styles = this.getDialog().getContentElement( 'advanced', 'advStyles' );
													styles && styles.updateStyle( 'background-color', this.getValue() );
												},
												setup : function( selectedTable )
												{
													this.setValue( selectedTable.getStyle( 'background-color' ) || '' );
												},
												commit : commitValue
											}
										]
									}
								]
							},
							{
								type : 'html',
								align : 'right',
								html : ''
							},
							{
								type : 'vbox',
								padding : 0,
								children :
								[
									{
										type : 'text',
										id : 'txtCaption',
										label : editor.lang.table.caption,
										setup : function( selectedTable )
										{
											this.enable();

											var nodeList = selectedTable.getElementsByTag( 'caption' );
											if ( nodeList.count() > 0 )
											{
												var caption = nodeList.getItem( 0 );
												var firstElementChild = caption.getFirst( CKEDITOR.dom.walker.nodeType( CKEDITOR.NODE_ELEMENT ) );

												if ( firstElementChild && !firstElementChild.equals( caption.getBogus() ) )
												{
													this.disable();
													this.setValue( caption.getText() );
													return;
												}

												caption = CKEDITOR.tools.trim( caption.getText() );
												this.setValue( caption );
											}
										},
										commit : function( data, table )
										{
											if ( !this.isEnabled() )
												return;

											var caption = this.getValue(),
												captionElement = table.getElementsByTag( 'caption' );
											if ( caption )
											{
												if ( captionElement.count() > 0 )
												{
													captionElement = captionElement.getItem( 0 );
													captionElement.setHtml( '' );
												}
												else
												{
													captionElement = new CKEDITOR.dom.element( 'caption', editor.document );
													if ( table.getChildCount() )
														captionElement.insertBefore( table.getFirst() );
													else
														captionElement.appendTo( table );
												}
												captionElement.append( new CKEDITOR.dom.text( caption, editor.document ) );
											}
											else if ( captionElement.count() > 0 )
											{
												for ( var i = captionElement.count() - 1 ; i >= 0 ; i-- )
													captionElement.getItem( i ).remove();
											}
										}
									},
									{
										type : 'text',
										id : 'txtSummary',
										label : editor.lang.table.summary,
										setup : function( selectedTable )
										{
											this.setValue( selectedTable.getAttribute( 'summary' ) || '' );
										},
										commit : function( data, selectedTable )
										{
											if ( this.getValue() )
												selectedTable.setAttribute( 'summary', this.getValue() );
											else
												selectedTable.removeAttribute( 'summary' );
										}
									}
								]
							}
						]
					}
					,dialogadvtab && dialogadvtab.createAdvancedTab( editor )
				]
			};
		}

		CKEDITOR.dialog.add( 'eptable', function( editor )
			{
				return tableDialog( editor, 'eptable' );
			} );
		CKEDITOR.dialog.add( 'eptableProperties', function( editor )
			{
				return tableDialog( editor, 'eptableProperties' );
			} );
	})();

	return CKEDITOR;

});
