/*globals define*/

define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!ep/ui/textedit-test',
    'jquery',
    'ep/ui/textedit',

    '$ready!'], function (QUnit, fixture, $, ep) {

    QUnit.module("ep/ui/textedit", {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest("ep/ui/textedit", function () {

        // create uiTextedit examples
        var textedit1 = ep('#textedit1').uiTextedit({
                action: 'Save'
            }),
            textedit2 = ep('#textedit2').uiTextedit({
                action: 'Save',
                onAfterClose:       function (input) {return false; },
                onBeforeOpen:       function (input) {return false; },
                onAfterOpen:        function (input) {return false; },
                convertLineBreaks:  true,
                stripHTML:          true,
                type: 'textarea'
            }),
            textedit3 = ep('#textedit3').uiTextedit({
                action: 'Save',
                additional: {CurrencyID: '123456'},
                name: 'myTestName',
                objectId: 815
            }),
            textedit4 = ep('#textedit4').uiTextedit({
                type: 'textarea',
                action: 'Save',
                validate: {required: true}
            });

        // start testing
        QUnit.equal(
            textedit1.uiTextedit('Instance') instanceof $.Widget,
            true,
            "uiTextedit widget was created and is instance of jQuery.Widget"
        );

        $.each(['_create', '_build', '_close', '_onkeypress', '_onkeyup', '_onkeyup', '_onmouseover', '_onmouseout', '_writeContentBack', '_saveContent', 'open', 'destroy'], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.uiTextedit.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );
        });

        textedit1.trigger('click');

        QUnit.equal(
            textedit1.siblings('input:visible').length,
            1,
            'Open Textedit onclick'
        );

        textedit1.siblings('input:visible').trigger('blur');

        QUnit.equal(
            textedit1.siblings('input:visible').length,
            0,
            'Close Textedit onblur'
        );

        textedit1.trigger('mouseover');

        QUnit.equal(
            textedit1.hasClass('ep-uiTextedit-Hover'),
            true,
            'Check mouseover'
        );

        textedit1.trigger('mouseout');

        QUnit.equal(
            textedit1.hasClass('ep-uiTextedit-Hover'),
            false,
            'Check mouseout'
        );

        var oldContent = textedit1.html();

        textedit1.trigger('click');

        var evt = new $.Event('keypress');
        evt.keyCode = 13;

        textedit1.siblings('input:visible')
            .val('myTestString')
            .trigger(evt);

        textedit1.trigger('click');

        QUnit.equal(
            textedit1.siblings('input:visible').val(),
            'myTestString',
            'Get the value of the first Textedit'
        );

        textedit2.trigger('click');

        QUnit.equal(
            textedit2.siblings('textarea:visible').length,
            1,
            'Open Textedit onclick'
        );

        textedit2.siblings('textarea:visible').trigger('blur');

        QUnit.equal(
            textedit2.siblings('textarea:visible').length,
            0,
            'Close Textedit onblur'
        );

        textedit2.uiTextedit('open');

        QUnit.equal(
            textedit2.siblings('textarea:visible').length,
            1,
            'Open Textedit by open method'
        );

        oldContent = textedit3.html();
        textedit3.trigger('click');
        textedit3.siblings('textarea:visible').val('Affe');

        evt = new $.Event('keypress');
        evt.keyCode = $.ui.keyCode.ESCAPE;

        textedit3.siblings('textarea:visible').trigger(evt);

        textedit4.uiTextedit('destroy');

        QUnit.equal(
            textedit4.data('epUiTextedit'),
            undefined,
            'uiTextedit widget was destroyed'
        );

        QUnit.start();
    });
});